var AS3 = (function()
{
	var queueDelay = 10;
	var lastCallTime = new Date().getTime();
	var queue = [];
	var queueTimeout = null;

	var queueCall = function(method, params)
	{
		queue.push({method: method, params: params});

		if(queueTimeout == null)
		{
			queueTimeout = setTimeout(handleQueue, queueDelay);
		}
	};


	var handleQueue = function()
	{
		queueTimeout = null;

		// No queued items to handle
		if(queue.length == 0)
		{
			return;
		}

		// Handle call
		var item = queue.shift();
		handleCall(item.method, item.params);

		// Handle the next queued item
		if(queue.length > 0)
		{
			queueTimeout = setTimeout(handleQueue, queueDelay);
		}

	};


	var handleCall = function(method, params)
	{
		lastCallTime = new Date().getTime();
		document.location.href = 'as:' + method + ':' + JSON.stringify(params);
	};


	var urlStringToObject = function(string)
	{
		var result = {};
		var parts = string.split('&');
		for(var i = 0; i < parts.length; i++)
		{
			var subParts = parts[i].split('=');
			result[subParts[0]] = subParts[1];
		}

		return result;
	};


	return {

		// -----------------------------------
		// Calls a method in Actionscript or queues it for later handling.
		//
		// @param method [String] Name of the method
		// @param params [Object] Optional method parameters
		// -----------------------------------
		call: function(method, params)
		{
			// When we the last call made?
			var lastCallDiff = new Date().getTime() - lastCallTime;

			if(params == undefined)
			{
				params = {};
			}

			// Insert scroll position and page height
			params.scrollY = $(window).scrollTop();
			params.pageHeight = $('#page').height();

			// If the call method was made in the last 100 milliseconds or we are already waiting on the queue,
			// add this call to the queue
			if(queue.length > 0 || lastCallDiff < 100)
			{
				queueCall(method, params);
			}
			// Handle the call right away
			else
			{
				handleCall(method, params);
			}
		},


		// -----------------------------------
		// Clears all queued method calls.
		// -----------------------------------
		clearAll: function()
		{
			queue = [];
			clearTimeout(queueTimeout);
			queueTimeout = null;
		},


		// -----------------------------------
		// Intercept link clicks and pass them through the call method instead
		// -----------------------------------
		interceptLinks: function()
		{
			$('body').on('click', 'a', function()
			{
				var href = $(this).attr('href');
				var method;
				var params = undefined;

				if(href.indexOf('as:') == 0)
				{
					href = href.substr(3);

					// Additional parameters?
					var tmpIndex = href.indexOf(':');
					if(tmpIndex >= 0)
					{
						method = href.substr(0, tmpIndex);
						params = urlStringToObject(href.substr(tmpIndex + 1));
					}
					else
					{
						method = href;
					}

					AS3.call(method, params);
					return false;
				}
			});

		}

	}

})();
