var Options = (function(){

	var pages = [];
	var currentPage = null;
	var optionValues = {};


	function updateOptions()
	{
		for(var i = 0; i < pages.length; i++)
		{
			pages[i].update(optionValues);
		}

		// Developer mode?
		if(optionValues.testing)
		{
			$('.developer').show();
		}

		// Current version
		if(optionValues.version)
		{
			$('.section.version').text('v' + optionValues.version);
		}
	}


	return {

		init: function()
		{
			// Setup pages
			pages.push(new OptionsHome());
			pages.push(new OptionsTray());
			pages.push(new OptionsBackground());
			pages.push(new OptionsReminder());
			pages.push(new OptionsDeveloper());

			// Setup back buttons
			$('.back').on('click', function()
			{
				Options.loadSection('home');
			});

			// Display the initial screen
			currentPage = $('#home');
			currentPage.show();
		},


		update: function(values)
		{
			optionValues = JSON.parse(values);
			updateOptions();
		},



		loadSection: function(section)
		{
			currentPage.hide();
			currentPage = $('#' + section);
			currentPage.show();

			window.scrollTo(0, 0);

			AS3.call('clickSound');
		},


		setOption: function(name, value)
		{
			AS3.call('setOption', {name: name, value: value});

			if(optionValues[name] == undefined || optionValues[name] != value)
			{
				optionValues[name] = value;
				updateOptions();
			}
		}


	};


})();


// -----------------------------------------
// Home
// -----------------------------------------
function OptionsHome()
{
	var $element = $('#home');

	var soundsToggle = new Toggle($element.find('.soundsToggle'));
	soundsToggle.onChange = function(value)
	{
		Options.setOption('sounds', value);
	};

	var ghostToggle = new Toggle($element.find('.ghostToggle'));
	ghostToggle.onChange = function(value)
	{
		Options.setOption('ghostImage', value);
	};

	var borderToggle = new Toggle($element.find('.borderToggle'));
	borderToggle.onChange = function(value)
	{
		Options.setOption('visibleBorder', value);
	};

	var largerPiecesToggle = new Toggle($element.find('.largerPiecesToggle'));
	largerPiecesToggle.onChange = function(value)
	{
		Options.setOption('largerPieces', value);
	};

	var updatesToggle = new Toggle($element.find('.updatesToggle'));
	updatesToggle.onChange = function(value)
	{
		Options.setOption('notifications', value);
	};


	// -----------------
	// Update
	// -----------------
	this.update = function(values)
	{
		soundsToggle.setValue(values.sounds == undefined ? true : values.sounds, false);
		ghostToggle.setValue(values.ghostImage == undefined ? false : values.ghostImage, false);
		borderToggle.setValue(values.visibleBorder == undefined ? false : values.visibleBorder, false);
		largerPiecesToggle.setValue(values.largerPieces == undefined ? true : values.largerPieces, false);
		updatesToggle.setValue(values.notifications == undefined ? false : values.notifications, false);

		$element.find('.trayOption').html(values.tray == undefined || !values.tray ? 'off' : 'on');
		$element.find('.dailyReminderOption').html(values.dailyReminder == undefined || !values.dailyReminder ? 'off' : 'on');
		//$element.find('.emailsOption').html(values.emails == undefined || !values.emails ? 'off' : 'on');
	}
}


// -----------------------------------------
// Tray
// -----------------------------------------
function OptionsTray()
{
	var $element = $('#tray');

	var trayToggle = new Toggle($element.find('.trayToggle'));
	trayToggle.onChange = function(value)
	{
		Options.setOption('tray', value);
	};

	// -----------------
	// Backgrounds
	// -----------------
	var backgrounds = ['#252738', '#383a4c', '#222222', '#434343', '#6e6e6e', '#909092', '#bcbcbd', '#f4f4f6'];
	var backgroundHolder = $element.find('.backgrounds');

	for(var i = 0; i < backgrounds.length; i++)
	{
		var elem = $('<div class="background"></div>');
		elem.css('background-color', backgrounds[i]);
		elem.data('color', backgrounds[i]);
		elem.on('click', function()
		{
			var color = $(this).data('color');

			updateBackground(color);
			Options.setOption('trayBackground', color);
		});

		backgroundHolder.append(elem);
	}

	// -----------------
	// Update background
	// -----------------
	var updateBackground = function(color)
	{
		$element.find('.backgrounds .background').each(function()
		{
			$(this).removeClass('on');

			if($(this).data('color') == color)
			{
				$(this).addClass('on');
			}
		});

	};

	// -----------------
	// Update
	// -----------------
	this.update = function(values)
	{
		trayToggle.setValue(values.tray == undefined ? true : values.tray, false);

		if(values.trayBackground != undefined)
		{
			updateBackground(values.trayBackground, false);
		}
	}
}


// -----------------------------------------
// Background
// -----------------------------------------
function OptionsBackground()
{
	var $element = $('#background');

	var preloadImages = [];

	// -----------------
	// Backgrounds
	// -----------------
	var backgrounds = [	'#222222', '#434343', '#6e6e6e', '#909092', '#bcbcbd', '#fafafa',
						'#5486b3', '#5a90c1', '#bed6ec', '#cdddec', '#ecf2f7',
						'retina_wood',
						'low_contrast_linen',
						'bold_blue',
						'1-1',
						'2-1',
						'3-8-2',
						'4-17',
						'6-9',
						'3-8-4',
						'7-9',
						'8-12',
						'10-4',
						'11-5',
						'3-8-5',
						'12-7',
						'13-14-2'
	];

	var backgroundHolder = $element.find('.backgrounds');

	for(var i = 0; i < backgrounds.length; i++)
	{
		var elem = $('<div class="background"></div>');

		if(backgrounds[i].charAt(0) == '#')
		{
			elem.css('background-color', backgrounds[i]);
		}
		else
		{
			var image = new Image();
			image.src = 'images/backgrounds/' + backgrounds[i] + '.png';
			preloadImages.push(image);

			elem.css('background-image', 'url("images/backgrounds/' + backgrounds[i] + '.png")');
		}

		elem.data('color', backgrounds[i]);
		elem.on('click', function()
		{
			var color = $(this).data('color');

			updateBackground(color);
			Options.setOption('background', color);
		});

		backgroundHolder.append(elem);
	}

	// -----------------
	// Update background
	// -----------------
	var updateBackground = function(color)
	{
		$element.find('.backgrounds .background').each(function()
		{
			$(this).removeClass('on');

			if($(this).data('color') == color)
			{
				$(this).addClass('on');
			}
		});

	};

	// -----------------
	// Update
	// -----------------
	this.update = function(values)
	{
		if(values.background != undefined)
		{
			updateBackground(values.background, false);
		}
	}
}


// -----------------------------------------
// Reminder
// -----------------------------------------
function OptionsReminder()
{
	var $element = $('#reminder');

	var dailyToggle = new Toggle($element.find('.dailyToggle'));
	dailyToggle.onChange = function(value)
	{
		Options.setOption('dailyReminder', value);
	};

	// -----------------
	// Update
	// -----------------
	this.update = function(values)
	{
		dailyToggle.setValue(values.dailyReminder == undefined ? false : values.dailyReminder, false);

		if(values.reminderTime != undefined)
		{
			$element.find('.reminderTime').html(values.reminderTime);
		}
	}
}


// -----------------------------------------
// Developer
// -----------------------------------------
function OptionsDeveloper()
{
	var $element = $('#developer');
	var _this = this;

	var cheatToggle = new Toggle($element.find('.cheatToggle'));
	cheatToggle.onChange = function(value)
	{
		Options.setOption('cheatMode', value);
	};

	var screenshotsToggle = new Toggle($element.find('.screenshotToggle'));
	screenshotsToggle.onChange = function(value)
	{
		Options.setOption('allowScreenshots', value);
	};

	var freeToggle = new Toggle($element.find('.freeToggle'));
	freeToggle.onChange = function(value)
	{
		Options.setOption('freeIAP', value);
	};

	$element.find('.production-server').on('click', function()
	{
		Options.setOption('server', 'production');
		_this.updateServer('production');
	});

	$element.find('.staging-server').on('click', function()
	{
		Options.setOption('server', 'staging');
		_this.updateServer('staging');
	});

	$element.find('.development-server').on('click', function()
	{
		Options.setOption('server', 'development');
		_this.updateServer('development');
	});


	// -----------------
	// Update
	// -----------------
	this.update = function(values)
	{
		cheatToggle.setValue(values.cheatMode == undefined ? true : values.cheatMode, false);
		screenshotsToggle.setValue(values.allowScreenshots == undefined ? true : values.allowScreenshots, false);
		freeToggle.setValue(values.freeIAP == undefined ? true : values.freeIAP, false);

		$element.find('.version').html(values.version);
		$element.find('.releaseStage').html(values.releaseStage);
		$element.find('.platform').html(values.platform);
		$element.find('.uuid').html(values.uuid);

		this.updateServer(values.server);
	};


	this.updateServer = function(value)
	{
		$element.find('a.active').removeClass('active');
		$element.find('.' + value + '-server').addClass('active');
	};


}